#!/usr/bin/python
# world.py

import sys, os
import subprocess       # consider replacing all calls to os.system with this
import ConfigParser
import shutil

import messages
from lab import XenLab
from vm import *

class XenWorld(XenLab):
	
	# __init__ #{{{
	# XenWorld extends XenLab -> user/group-specific
	# arguments
	# uname		username
	# path		path to lab directory 
	def __init__ (self, uname, path):
		
		# initalize base class
		XenLab.__init__(self, path)

		# Derive groups-specific info from uname, path, and XenLab
		self.user = uname
		self.groupNo = self.getUserGroup(self.user)
                if self.groupNo is None:
                    return None
		self.worldPath = self.labPath + "/" + "group" + self.groupNo

		# Check here - does worldPath exist? If no, we gotta create it
		if os.path.isdir(self.worldPath):
			messages.info("Found world directory %s. Continuing." % \
					self.worldPath)
		else:
			messages.info("Directory %s does not exist. Creating." % \
					self.worldPath)
			self.createDirectoryStructure()

		# Continue, now that everything is in it's right place

		# World info
		self.name = ""					# generated by this class
		self.configFile = ""
		self.configObject = ""
		self.bridge = ""

		# Node info
		self.numberOfNodes = 0
		self.vmList = []
		
		# Get config information from configFile
		self.configFile = self.worldPath + "/" + "world.cfg"
		self.setVariablesFromWorldConfigFile()

		# Generate a name
		# classNo-labNo-groupNo
		self.name = "%s-%s-%s" \
				% (self.classNo, self.labNo, self.groupNo)

		# Create bridge	
		self.bridge = self.generateBridgeName()
		self.createBridge()

		for i in range(int(self.numberOfNodes)):
			vm = VirtualMachine(self.classNo, \
					self.labNo, \
					self.groupNo, \
					str(i), \
					self.bridge, \
					self.configObject, \
					self.worldPath)
			self.vmList.append(vm)

#}}}
	def __str__ (self):#{{{
		return "User: %s\nC%s, Lab%s, Group%s\nWorld Path: %s" \
				% (self.user, \
				self.classNo, self.labNo, self.groupNo,\
				self.worldPath)

#}}}
	def readConfigFile (self, wcf):#{{{
		config = ConfigParser.ConfigParser()
		config.read(wcf)

		return config

#}}}
	def setVariablesFromWorldConfigFile (self):#{{{
		self.configObject = self.readConfigFile(self.configFile)
		try:
			self.numberOfNodes = int(self.configObject.get("World Data", \
					"numberOfNodes"))
		except ConfigParser.NoSectionError:
			messages.error("Missing 'World Data' section in config file: " + \
					self.configFile)

#}}}
	# createDirectoryStructure(self)#{{{
	# creates group# directory in the lab directory.
	# copies world file from base, changing disk paramter.
	# copies images files to group# dir.
	def createDirectoryStructure (self):
		messages.info("Create directory world structure at " + self.worldPath)
		
		# create configParser for base/world.cfg
		baseConfig = ConfigParser.ConfigParser()
		baseConfig.optionxform=str			# to preserve case
		baseConfig.read(self.labPath + "/base/" + "world.cfg")

		# Create the worldPath directory (group directory)
		if not os.path.isdir(self.worldPath):
			os.mkdir(self.worldPath)

		# Create matching lists for base storage files -> world
		bList = []
		wList = []

		# Copy world.cfg, modifying pathnames from base -> group#
		numNodes = int(baseConfig.get("World Data", "numberOfNodes"))
		for i in range(numNodes):
			nodeSection = "Node " + str(i)
			nodeDisks = baseConfig.get(nodeSection, "disk").split(",")
			worldDisks = []
			for j in range(len(nodeDisks)):
				bPath = nodeDisks[j].strip()
				diskName = bPath.split("/")[-1]
				wPath = self.worldPath + "/" + diskName
				worldDisks.append(wPath)
				bList.append(bPath)
				wList.append(wPath)
			newNodeDisks = ','.join(worldDisks)
			baseConfig.set(nodeSection, "disk", newNodeDisks)

		newWorldFile = self.worldPath + "/" + "world.cfg"
		configFile = open(newWorldFile, 'w') 
		baseConfig.write(configFile)

		# Copy storage files
		for i in range(len(bList)):
			#shutil.copyfile(bList[i], wList[i])
                        messages.debug("COPY start: " + bList[i])
			subprocess.check_call(["cp","--sparse=always",bList[i], wList[i]])
                        messages.debug("COPY end:   " + wList[i])



#}}}
	# Bridge functions#{{{
	#
        # Rewriting these 8/2
        # to follow protocol in "Running Xen," p. 386
        #
	def startBridge (self):
		command = "sudo ip link set " + self.bridge + " up"
		messages.info("Bringing up bridge: " + command)
                #p = Popen(command, shell=True)
                #sts = os.waitpid(p.pid, 0)[1]
                os.system(command)

	def stopBridge (self):
		command = "sudo ifconfig " + self.bridge + " down"
		messages.info("Tearing down bridge: " + command)
		os.system(command)

	def destroyBridge(self):
		self.stopBridge()

		command = "sudo /usr/sbin/brctl delbr " + self.bridge
		result = os.system(command)

		if result == 0:
			messages.info("Bridge %s removed successfully." % self.bridge)
		else:
			messages.error("Unknown error occurred: %s" % str(result))

	
	# TODO: only execute addbr if bridge doesn't yet exist
	def createBridge (self):
            # check to see if bridge exists
            command = "sudo /usr/sbin/brctl show | grep " + self.bridge + " > /dev/null"
            result = os.system(command)         # exit status 0 if bridge is up, 1 o.w.

            if result == 0:            # Bridge already exists, bring it up
                messages.info("Bridge %s already exists." % self.bridge)
                self.startBridge()
            else:                      # Create bridge, then bring it up
                command = "sudo /usr/sbin/brctl addbr " + self.bridge
                result = os.system(command)
                if result == 0:		# Bridge was created, now bring it up
                        messages.info("Bridge %s added successfully." % self.bridge)
                        self.startBridge()
                else:
                        messages.error("Error creating bridge: " + str(result))


	def generateBridgeName (self):
		brname = "br-%s" % (self.name)
		messages.debug("Generated bridge name: " + brname)

		return brname
#}}}
# main()#{{{
# for debugging purposes only
def main():
	labdir = '/home/xenlabs/labs/cs343/lab0' 
	username = 'grobinso'
	test = XenWorld(username, labdir)
	print test

if __name__ == "__main__":
	main()#}}}

# vim:fdm=marker
